#include <linux/version.h>

#include <linux/string.h>
#include <linux/errno.h>
#include <linux/module.h>

#include "nikal.h"

#ifdef nNIKAL100_kCopyrightString
   MODULE_LICENSE(nNIKAL100_kCopyrightString);
#endif

#ifdef nNIKAL100_kAuthorString
   MODULE_AUTHOR(nNIKAL100_kAuthorString);
#endif

#ifdef nNIKAL100_kDescriptionString
   MODULE_DESCRIPTION(nNIKAL100_kDescriptionString);
#endif


#include <linux/pci.h>
#include <linux/mod_devicetable.h>

int nNIAPAL200_driverEntry(struct nNIKAL100_tDriver *driver);

static struct nNIKAL100_tDriver atomicDriver;
static void* nativeDriverRegistration;

/* native driver registration functions. These functions are auto-generated */
void *registerNativeDriver(nNIKAL100_tDriver*);
void unregisterNativeDriver(void*);

int init_module(void)
{
   atomicDriver.module = THIS_MODULE;

   if (nNIAPAL200_driverEntry(&atomicDriver))
      return -ENODEV;

   nativeDriverRegistration = registerNativeDriver(&atomicDriver);

   if (nativeDriverRegistration == NULL)
   {
      if (atomicDriver.unload)
         atomicDriver.unload(&atomicDriver);
      return -ENODEV;
   }

   return 0;
}

void cleanup_module(void)
{
   unregisterNativeDriver(nativeDriverRegistration);

   if (atomicDriver.unload)
      atomicDriver.unload(&atomicDriver);
}

static struct pci_device_id nipciidtable[] = {
   {0x1093, 0x74DA, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x74D9, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7410, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7411, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7517, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x740E, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x740F, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7515, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x74C3, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x74C4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x74C1, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x74C2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70ED, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70E9, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70E8, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70EC, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70E3, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70DE, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7081, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70E6, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70E2, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70DD, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x7080, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70EF, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70EB, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70EE, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70EA, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70E5, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70E0, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70DC, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70E7, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70E4, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70DF, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
   {0x1093, 0x70DB, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&atomicDriver},
    { },
};
MODULE_DEVICE_TABLE(pci, nipciidtable);

static struct pci_driver niPciDriver = {
   .name = "ni843x",
   .id_table = nipciidtable,
};

void *registerNativeDriver(nNIKAL100_tDriver* ignore)
{
   return nNIKAL200_registerPCIDriver(&niPciDriver);
}
void unregisterNativeDriver(void* data)
{
   nNIKAL200_unregisterPCIDriver(data);
}


#define nNIKAL100_mExportSymbol EXPORT_SYMBOL
